const assert = require('assert');
const { Given, When, Then } = require('@cucumber/cucumber');

const { fixtures } = require('../../fixtures/server');

const mockValidConfig = fixtures.get('config-valid');
const mockInvalidConfig = fixtures.get('config-invalid');
const mockSchema = require('../../mock/swagger.json');
// const { Adapter } = require('../../../dist/adapter');

Given('valid config', function () {
  this.config = mockValidConfig;
});

Given('invalid config', function () {
  this.schema = mockSchema;
  this.config = mockInvalidConfig;
});

When('the adapter instance is created', function () {
  this.adapter = new this.classes.TrackedAdapter(this.protocol, this.config);
  this.adapter.setupCallTracking(this.recorder, this.tracker);
});

Then(
  'a successful response is returned with status {int}',
  function (expectedStatus) {
    this.call.then((response) => {
      assert.equal(response.status, expectedStatus);
    });
  }
);

Then(
  'an error response is returned with status {int}',
  function (expectedStatus) {
    this.call.catch((error) => {
      assert.equal(error.response.status, expectedStatus);
    });
  }
);

Then('the credentials are deleted', function () {
  assert.equal(this.adapter.credentials, null);
});

Then('the credentials are not deleted', function () {
  assert.notEqual(this.adapter.credentials, null);
});

When('the {string} method call counts are checked', function (method) {
  let newCallCount = this.adapter.getTrackedCalls(method).length;
  this.callCounts[method] = newCallCount;
});

Then('the {string} method is not called again', function (method) {
  let newCallCount = this.adapter.getTrackedCalls(method).length;
  assert.equal(this.callCounts[method], newCallCount);
});

Then(
  'the total number of calls to {string} was {int}',
  function (method, expectedCallCount) {
    let callCount = this.adapter.getTrackedCalls(method).length;
    assert.equal(callCount, expectedCallCount);
  }
);

Then(
  'the total number of {string} requests to {string} was {int}',
  function (method, endpoint, expectedCallCount) {
    let url = `${this.api}${endpoint}`;
    let requestHistory = this.mockAxios.history[method.toLowerCase()].filter(
      (request) => request.url === url
    );
    assert.equal(requestHistory.length, expectedCallCount);
  }
);
